/* THIS FILE IS AUTOGENERATED BY gen-usercounters.py - DO NOT EDIT */

#ifndef USE_COUNTER_DOM_METHOD
#define USE_COUNTER_DOM_METHOD(interface_, name_) // nothing
#define DEFINED_USE_COUNTER_DOM_METHOD
#endif


#ifndef USE_COUNTER_DOM_ATTRIBUTE
#define USE_COUNTER_DOM_ATTRIBUTE(interface_, name_) // nothing
#define DEFINED_USE_COUNTER_DOM_ATTRIBUTE
#endif


#ifndef USE_COUNTER_CSS_PROPERTY
#define USE_COUNTER_CSS_PROPERTY(interface_, name_) // nothing
#define DEFINED_USE_COUNTER_CSS_PROPERTY
#endif


#ifdef DEFINED_USE_COUNTER_DOM_METHOD
#undef DEFINED_USE_COUNTER_DOM_METHOD
#undef USE_COUNTER_DOM_METHOD
#endif


#ifdef DEFINED_USE_COUNTER_DOM_ATTRIBUTE
#undef DEFINED_USE_COUNTER_DOM_ATTRIBUTE
#undef USE_COUNTER_DOM_ATTRIBUTE
#endif


#ifdef DEFINED_USE_COUNTER_CSS_PROPERTY
#undef DEFINED_USE_COUNTER_CSS_PROPERTY
#undef USE_COUNTER_CSS_PROPERTY
#endif

