/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

/**
 * Legacy constants for specifying programming languages.
 *
 * JAVASCRIPT is needed to avoid breaking addons that use it in nsIClassInfo
 * to define fields that are no longer needed.
 *
 * UNKNOWN and JAVASCRIPT are also used in implementations of
 * nsIStackFrame::language.
 */

[scriptable, uuid(02ad9f22-3c98-46f3-be4e-2f5c9299e29a)]
interface nsIProgrammingLanguage : nsISupports
{
    const uint32_t UNKNOWN     = 0;
    // 1 is unused.
    const uint32_t JAVASCRIPT  = 2;
};
